<?php
namespace Models;

// модель для чеков
class Receipts extends \DB\SQL\Mapper {
	public $db, $f3, $fr;

	public function __construct($db, $f3) {
		parent::__construct($db, 'receipts' ); // подключаемся к таблице чеков
		$this->db=$db;
		$this->f3=$f3;
		$this->fr=new \Models\Fr($db, $f3);
	}

	// Информация об открываемой странице
	function page_data($page='sales') {
		$pages=array(
			'sales'=>array(
				'title'=>'Продажа',
				'inc'=>'receipts_sales.htm'
			),
			'return'=>array(
				'title'=>'Возврат',
				'inc'=>'receipts_return.htm'
			),
			'search'=>array(
				'title'=>'Поиск чеков',
				'inc'=>'receipts_search.htm'
			),
			'paymentmethods'=>array(
				'title'=>'Оплата чека',
				'inc'=>'receipts_paymentmethods.htm'
			),
			'show'=>array(
				'title'=>'Просмотр чека',
				'inc'=>'receipts_show.htm'
			),
			'change_price'=>array(
				'title'=>'Редактирование цены продажи',
				'inc'=>'receipts_change_price.htm'
			),
			'cashin'=>array(
				'title'=>'Внесение денег в кассу',
				'inc'=>'receipts_cashin.htm'
			),
			'cashout'=>array(
				'title'=>'Изъятие денег из кассы',
				'inc'=>'receipts_cashout.htm'
			),
			'reports'=>array(
				'title'=>'Смены и отчеты',
				'inc'=>'receipts_reports.htm'
			),
			'new'=>array(
				'title'=>'Открытие нового чека',
				'inc'=>'receipts_new.htm'
			),
			'switch'=>array(
				'title'=>'Переключение на другой чек',
				'inc'=>'receipts_switch.htm'
			),
			'all_open'=>array(
				'title'=>'Открытые чеки',
				'inc'=>'receipts_all_open.htm'
			),
			'move'=>array(
				'title'=>'Перемещение чека на другой стол',
				'inc'=>'receipts_move.htm'
			),
			'online_order'=>array(
				'title'=>'Поиск заказов из интернет-магазина',
				'inc'=>'receipts_online_order.htm'
			),
			'online_order_result'=>array(
				'title'=>'Заказы из интернет-магазина',
				'inc'=>'receipts_online_order_result.htm'
			),
			'print'=>array(
				'title'=>'Печать чека',
				'inc'=>'receipts_print.htm'
			),
			'walk'=>array(
				'title'=>'Отметить проход сотрудника',
				'inc'=>'receipts_walk.htm'
			)
			,
			'walk_add_card'=>array(
				'title'=>'Отметить проход сотрудника',
				'inc'=>'receipts_walk_add_card.htm'
			)
		);
		$pages[$page]['page']=$page;
		return $pages[$page];
	}

	// Выбираем чеки
	function select_receipts($query=null, $options=null) {
		if(!$query) {
			$query=array('r.id>?', 0); // чтобы хоть что-то вставить в фильтры
		}

		// формируем сортировку и лимиты
		$option='';
		if($options['order']) {
			$option.=' ORDER BY '.$options['order'];
		}
		if($options['limit']) {
			$option.=' LIMIT '.$options['limit'];
		}
		// собственно сам запрос
		$receipts=array();
		$sql="SELECT r.*, u.name, u.short_name, g.title, t.title AS table_title, room.id AS room_id, room.title AS room_title FROM receipts AS r LEFT JOIN users AS u ON u.id=r.user_id LEFT JOIN usergroups AS g ON g.id=u.group_id LEFT JOIN tables AS t ON t.id=r.table_id LEFT JOIN rooms AS room ON room.id=t.room WHERE (".$query[0].") AND r.time>0 ".$option;
		$receipts=$this->db->exec($sql, $query[1]);
		$receiptlines=new \Models\Receiptlines($this->db, $this->f3);
		foreach ($receipts as $n => $r) {
			$receipts[$n]['receiptlines']=$receiptlines->select_lines(array('receipt_id=?', $r['id']));
			if($r['type']==$this->f3->get('receipt_sales') || $r['type']==$this->f3->get('receipt_return')) {
				if($receipts[$n]['lines_count']==null) {
					$receipts[$n]['lines_count']=$receipts[$n]['lines_count_add']=count($receipts[$n]['receiptlines']);
				}
			}
			else {
				unset($receipts[$n]['lines_count']);
			}
			$receipts[$n]['paymentmethods']=json_decode($r['payment'], true);
		}
		return $receipts;
	}

	// Узнаем окрытый чек
	function get_open_receipt($rm, $store, $open=true) {
		$this->reset();
		$user_id=$this->f3->get('SESSION.user.id');
		$receipt_id=$this->f3->get('SESSION.receipt_id');
		$table_id=$this->f3->get('SESSION.table_id');
		if(!$table_id) $table_id=0;
		if($table_id>0) {
			$t=new \Models\Tables($this->db, $this->f3);
			$table=$t->select_tables(array('id=?', $table_id), array('limit'=>1));
		}
		$this->load(array('rm=? AND store=? AND time=0 AND user_id=? AND table_id=?', array($rm, $store, $user_id, $table_id)));
		if($this->dry()) {
			if(!$open) {
				return array(
					'success'=>false,
					'txt'=>'not_found'
				);
			}
			$receipt=array(
				'rm'=>$rm,
				'store'=>$store,
				'user_id'=>$user_id,
				'table_id'=>$table_id,
				'table'=>$table[0],
				'time'=>0
			);
			$this->copyFrom($receipt);
			if($this->insert()) {
				$receipt['id']=$this->get('_id');
				return array(
					'success'=>true,
					'txt'=>'save_successful',
					'receipt'=>$receipt,
					'open'=>array()
				);
			}
			else {
				return array(
					'success'=>false,
					'txt'=>'db_error'
				);
			}
		}
		else {
			$receipt=array();
			$open=array();
			do {
				$ticket=$this->cast();
				if($ticket['id']==$receipt_id) {
					$receipt=$ticket;
				}
				else {
					$open[]=$ticket;
				}
			}
			while($this->skip());
			if(empty($receipt)) {
				$receipt=$open[0];
			}
			if($receipt['partner_id']>0) {
				$p=new \Models\Partners($this->db, $this->f3);
				$partner=$p->select_partner(array('partner_id=?', $receipt['partner_id']), array('limit'=>1));
				$receipt['partner']=$partner[0];
			}
			$receipt['table']=$table[0];
			return array(
				'success'=>true,
				'receipt'=>$receipt,
				'open'=>$open
			);
		}
	}

	// Ищем чеки
	function search($id, $changes, $barcode, $dates, $type, $start=0, $limit=30) {
		$where='r.type!=?';
		$val=array($this->f3->get('receipt_walk'));
		if($id) {
			$where.=' AND r.id=?';
			$val[]=$id;
		}
		if($changes) {
			$where='r.changes=?';
			$val[]=$changes;
		}
		if($dates) {
			if($where!='')$where.=' AND ';
			$where.='r.time>=? AND r.time<=?';
			$val[]=$dates[0];
			$val[]=$dates[1]+3600*24;
		}
		if($type) {
			if($where!='')$where.=' AND ';
			$where.='r.type=?';
			$val[]=$type;
		}
		if($barcode) {
			if($where!='') $where.=' AND ';
			$p=new \Models\Products($this->db, $this->f3);
			$products=$p->search($barcode, null, null, true);
			if($products['success']) {
				$product_id=$products['products'][0]['id'];
				$receiptlines=new \Models\Receiptlines($this->db, $this->f3);
				$lines=$receiptlines->select_lines(array('product_id=?', $product_id));
				if($lines) {
					$bwhere='';
					foreach ($lines as $l) {
						if($bwhere!='')$bwhere.=' OR ';
						$bwhere.=' r.id=? ';
						$val[]=$l['receipt_id'];
					}
					if($bwhere!='') {
						$where.=' ('.$bwhere.') ';
					}
				}
				else {
					$where.='r.id=?';
					$val[]=-1;
				}
			}
			else {
				$where.='r.id=?';
				$val[]=-1;
			}
		}
		if(!$where) {
			$where='r.id>?';
			$val[]=0;
		}
		$receipts=$this->select_receipts(array($where, $val), array('order'=>'time DESC', 'limit'=>$start.', '.$limit));
		$sql="SELECT COUNT(*) AS count FROM receipts AS r WHERE ".$where;
		$res=$this->db->exec($sql, $val);
		$count=$res[0]['count'];

		return array(
			'success'=>true,
			'count'=>$count,
			'receipts'=>$receipts
		);
	}

	// Проверка правильности заполнения контактных данных покупателя
	function client_validate($client) {
		$result=false;
		if(mb_strpos($client, '@')===false) {
			// Проверяем телефон
			if(mb_substr($client, 0, 1)=='7') $client='+'.$client;
			if(mb_substr($client, 0, 1)=='8') {
				$client='+7'.mb_substr($client, 1);
			}
			if(preg_match("/\+79[0-9]{9}/", $client)) $result=true;
		}
		else {
			// проверяем e-mail
			if(filter_var($client, FILTER_VALIDATE_EMAIL))$result=true;
		}
		return $result;
	}

	// Пробиваем чек
	function add_ticket($receipt_id, $change, $type, $paymentmethod, $client_sum, $sum, $lines, $client, $discount=0, $partner_id=0, $sales_receipts=0, $print=1) {
		if($client && !$this->client_validate($client)) {
			return array(
				'success'=>false,
				'txt'=>'incorrect_email_or_phone'
			);
		}
		if(is_array($lines)) {
			$lines_count=count($lines);
		}
		else {
			$lines_count=0;
		}
		if($lines_count==0) {
			return array(
				'success'=>false,
				'txt'=>'lines_not_found'
			);
		}

		// Формируем данные оплаты
		$payment=array();
		foreach ($paymentmethod as $p_id) {
			$payment[]=array('method'=>$p_id, 'sum'=>$client_sum[$p_id]);
		}

		// формируем данные чека
		$receipt=array(
			'id'=>$receipt_id,
			//'user_id'=>$this->f3->get('SESSION.user.id'),
			'changes'=>$change['id'],
			'changes_data'=>json_encode($change),
			'time'=>microtime(true),
			'type'=>$type,
			'payment'=>json_encode($payment),
			'sum'=>$sum,
			'client'=>$client,
			'partner_id'=>$partner_id,
			'lines_count'=>$lines_count
		);
		
		$this->reset();
		$this->load(array('id=?', $receipt_id), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		$this->copyFrom($receipt);
		$this->fr->ticket($receipt, $lines, $discount, $sales_receipts, $print);
		if(!$this->update()) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}


		return array(
			'success'=>true,
			'txt'=>'save_successful'
		);
	}

	function ticket_return($rm, $store, $change, $post) {
		if(is_array($post['receiptlines'])) $lines_count=count($post['receiptlines']);
		else $lines_count=0;
		if($lines_count==0) {
			return array(
				'success'=>false,
				'txt'=>'lines_not_found'
			);
		}

		// Формируем данные оплаты
		$payment=array();
		foreach ($post['paymentmethod'] as $p_id) {
			$payment[]=array('method'=>$p_id, 'sum'=>$post['client_sum'][$p_id]);
		}

		// формируем данные чека
		$receipt=array(
			'user_id'=>$this->f3->get('SESSION.user.id'),
			'rm'=>$rm,
			'store'=>$store,
			'changes'=>$change['id'],
			'changes_data'=>json_encode($change),
			'time'=>0,
			'type'=>$post['type'],
			'payment'=>json_encode($payment),
			'sum'=>$post['sum'],
			'client'=>'',
			'partner_id'=>$post['partner_id'],
			'lines_count'=>$lines_count
		);
		$this->reset();
		$this->copyFrom($receipt);
		if(!$this->insert()) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
		$receipt['id']=$this->get('_id');

		return array(
			'success'=>true,
			'txt'=>'save_successful',
			'id'=>$receipt['id'],
			'receipt'=>$receipt
		);
	}

	// Проводим чек возврата
	function ticket_ready($receipt, $post) {
		$this->reset();
		$this->load(array('id=?', $receipt['id']), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		$this->time=microtime(true);
		$this->fr->ticket($receipt, $post['receiptlines'], $post['discount']);
		if($this->update()) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'function'=>'return_reset()'
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Чек смены
	function ticket_change($api_data, $change, $type, $paymentmethod, $sum=0) {
		$this->reset();
		// Формируем данные оплаты
		$payment[]=array('method'=>$paymentmethod, 'sum'=>$sum);
		// формируем данные чека
		$receipt=array(
			'store'=>$api_data['store_id'],
			'rm'=>$api_data['rm_id'],
			'user_id'=>$this->f3->get('SESSION.user.id'),
			'changes'=>$change['id'],
			'changes_data'=>json_encode($change),
			'time'=>microtime(true),
			'type'=>$type,
			'payment'=>json_encode($payment),
			'sum'=>$sum,
			'client'=>'',
			'partner_id'=>0
		);
		$this->copyFrom($receipt);
		if($this->insert()) { // сохраняем
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'id'=>$this->get('_id')
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Удаление чека
	function delete_receipt($id) {
		$this->reset();
		$this->load(array('id=?', $id), array('limit'=>1));
		$this->erase();
		$this->db->exec("DELETE FROM receiptlines WHERE receipt_id=?", $id);
	}

	// Внесение и изъятие денег из кассы
	function cashin($api_data, $change, $pay, $sum, $type, $create_receipt=0, $description='') {
		$this->fr->cashin($change['id'], $sum);
		$receipt=array(
			'store'=>$api_data['store_id'],
			'rm'=>$api_data['rm_id'],
			'user_id'=>$this->f3->get('SESSION.user.id'),
			'changes'=>$change['id'],
			'changes_data'=>json_encode($change),
			'time'=>microtime(true),
			'type'=>$type,
			'payment'=>json_encode(array(array('method'=>$pay, 'sum'=>$sum))),
			'sum'=>$sum,
			'client'=>'',
			'create_receipt'=>$create_receipt,
			'description'=>$description
		);
		$this->reset();
		$this->copyFrom($receipt);
		$this->save();
		if($this->get('_id')) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'form_reset'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Статистика по смене
	function change_statistic($change) {
		$receipts=$this->select_receipts(array('changes=? AND type!=? AND type!=?', array($change, $this->f3->get('receipt_change_open'), $this->f3->get('receipt_change_close'))));
		$report=array(
			$this->f3->get('receipt_sales')=>array('sum'=>0, 'count'=>0),
			$this->f3->get('receipt_return')=>array('sum'=>0, 'count'=>0),
			$this->f3->get('receipt_cashin')=>array('sum'=>0, 'count'=>0),
			$this->f3->get('receipt_cashout')=>array('sum'=>0, 'count'=>0),
			'receipt_remote'=>array('sum'=>0, 'count'=>0),
		);
		foreach ($receipts as $r) {
			$report[$r['type']]['count']+=1;
			$report[$r['type']]['sum']+=$r['sum'];
			$pays=json_decode($r['payment'], true);
			foreach ($pays as $p) {
				if($p['sum']>$r['sum']) {
					$p['sum']=$r['sum'];
				}
				$report[$r['type']]['pay'][$p['method']]+=$p['sum'];
				if($r['remote']>0) {
					$report['receipt_remote']['pay'][$p['method']]+=$p['sum'];
					$report['receipt_remote']['count']+=1;
					$report['receipt_remote']['sum']+=$r['sum'];
				}
			}
		}
		return $report;
	}

	// Отчет по секциям
	function rep_sections() {
		$fr_type=$this->fr->get_fr()['type'];
		if($fr_type==1) {
			$this->fr->rep_sections();
			return array(
				'success'=>true,
				'txt'=>'report_is_ready'
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'fr_only'
			);
		}
	}

	// Печать копии последнего чека
	function print_last() {
		if($this->fr->sets['active']) {
			$this->fr->c();
			return array(
				'success'=>true,
				'txt'=>'request_has_been_sent'
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'kkm_is_not_activity'
			);
		}
	}

	// Открытие нового пустого чека
	function open_new($rm, $store) {
		$user_id=$this->f3->get('SESSION.user.id');
		$table_id=$this->f3->get('SESSION.table_id');
		$this->reset();
		$receipt=array(
			'rm'=>$rm,
			'store'=>$store,
			'user_id'=>$user_id,
			'table_id'=>$table_id,
			'time'=>0
		);
		$this->copyFrom($receipt);
		if($this->insert()) {
			$this->f3->set('SESSION.receipt_id', $this->get('_id'));
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'reload'=>'.receiptlines',
				'reload_menu'=>true,
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Переключение на другой чек
	function switch_receipt($id) {
		if(!$id) {
			return array(
				'success'=>false,
				'txt'=>'not_enough_data'
			);
		}
		$this->reset();
		$this->load(array('id=?', $id), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}

		$this->f3->set('SESSION.receipt_id', $this->id);
		$this->f3->set('SESSION.table_id', $this->table_id);
		return array(
			'success'=>true,
			'txt'=>'save_successful',
			'reload'=>'.receiptlines',
			'reload_menu'=>true,
			'close_all_window'=>true
		);
	}

	// Добавление клиента к чеку
	function add_partner($receipt_id, $partner) {
		$this->reset();
		$this->load(array('id=? AND time=0', $receipt_id), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		$this->partner_id=$partner['partner_id'];
		if($partner['email']) {
			$contact=$partner['email'];
		}
		else {
			$contact=$partner['phone'];
		}
		$this->client=$contact;
		$this->discount=$partner['discount'];
		if($this->update()!==false) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'reload'=>'.receiptlines',
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Убрать клиента из чека
	function remove_partner($id) {
		$this->reset();
		$this->load(array('id=?', $id), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		$this->partner_id='';
		$this->discount=0;
		if($this->update()) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'reload'=>'.receiptlines',
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Сохранение произвольной скидки в чек
	function set_discount($discount) {
		if(is_numeric($discount) && $discount>=0 && $discount<=100) {
			$this->reset();
			$id=$this->f3->get('SESSION.receipt_id');
			$this->load(array('id=?', $id), array('limit'=>1));
			if($this->dry()) {
				return array(
					'success'=>false,
					'txt'=>'not_found'
				);
			}
			$this->discount=$discount;
			if($this->update()!==false) {
				return array(
					'success'=>true,
					'txt'=>'save_successful'
				);
			}
			else {
				return array(
					'success'=>false,
					'txt'=>'db_error'
				);
			}
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'incorrect_discount'
			);
		}
	}

	// Поиск всех открытых чеков
	function all_open($start=0, $limit=30) {
		$sql="SELECT r.*, u.name, u.short_name, g.title, SUM(l.count) AS lines_count, SUM(CASE l.vat_type WHEN 0 THEN l.price*l.count WHEN 1 THEN (l.price+(l.price/100*l.vat))*l.count END) AS lines_sum, CASE WHEN l.receipt_id>0 THEN l.receipt_id ELSE r.id END AS r_id, rooms.title AS room_title, rooms.id AS room_id, t.title AS table_title FROM receipts AS r LEFT JOIN users AS u ON u.id=r.user_id LEFT JOIN usergroups AS g ON g.id=u.group_id LEFT JOIN receiptlines AS l ON receipt_id=r.id LEFT JOIN tables AS t ON t.id=r.table_id LEFT JOIN rooms ON rooms.id=t.room WHERE r.time=0 GROUP BY r_id LIMIT ".$start.', '.$limit;
		$receipts=$this->db->exec($sql);
		$sql="SELECT COUNT(*) AS count FROM receipts AS r WHERE time=0";
		$res=$this->db->exec($sql);
		$count=$res[0]['count'];
		return array(
			'success'=>true,
			'count'=>$count,
			'receipts'=>$receipts
		);
	}

	// Перехват чека
	function intercept($rm, $store, $id) {
		$this->reset();
		$this->load(array('id=? AND time=0', $id), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		$this->rm=$rm;
		$this->store=$store;

		$this->user_id=$this->f3->get('SESSION.user.id');
		$sets=$this->f3->get('base_settings');
		if($sets['cafe_mode']==1) {
			if($this->table_id==0) $this->table_id=$this->f3->get('SESSION.table_id');
		}
		else {
			$this->table_id=0;
		}
		if($this->update()!==false) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'reload'=>'.receiptlines',
				'reload_menu'=>true,
				'receipt'=>$this->cast(),
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Получаем открытый пустой чек
	function get_empty($rm, $store) {
		$sql="SELECT r.id, COUNT(l.id) AS count FROM `receipts` AS `r` LEFT JOIN `receiptlines` AS `l` ON l.receipt_id=r.id WHERE r.time=0 AND r.rm=? AND r.store=? GROUP BY (r.id) ORDER BY r.id";
		$val=array($rm, $store);
		$res=$this->db->exec($sql, $val);
		if($res) {
			foreach ($res as $r) {
				if($r['count']==0) {
					return $r['id'];
				}
			}
		}
		return false;
	}

	// Получаем открытые не пустые чеки
	function get_not_empty($rm, $store) {
		$sql="SELECT COUNT(r.id) AS count FROM `receipts` AS `r` JOIN `receiptlines` AS `l` ON l.receipt_id=r.id WHERE r.time=0 AND r.rm=? AND r.store=? GROUP BY (l.receipt_id)";
		$val=array($rm, $store);
		$res=$this->db->exec($sql, $val);
		return $res[0]['count'];
	}
	
	// Получаем открытые не пустые чеки для стола
	function get_not_empty_for_table($table_id) {
		$sql="SELECT r.*, SUM(l.count) AS lines_count, CASE WHEN l.receipt_id>0 THEN l.receipt_id ELSE r.id END AS r_id FROM receipts AS r JOIN receiptlines AS l ON receipt_id=r.id WHERE r.time=0 AND r.table_id=? GROUP BY r_id";
		$val=array($table_id);
		$res=$this->db->exec($sql, $val);
		return $res;
	}

	// Удаляем открытые чеки
	function delete_open($rm, $store) {
		// Список открытых чеков
		$receipts=$this->db->exec("SELECT `id` FROM `receipts` WHERE `time`=0 AND `rm`=? AND `store`=?", array($rm, $store));
		if(!empty($receipts)) {
			$in='';
			$val=array();
			foreach ($receipts as $r) {
				if(!empty($val)) {
					$in.=',';
				}
				$in.='?';
				$val[]=$r['id'];
			}
			$sql="DELETE FROM `receipts` WHERE `id` IN (".$in.")";
			$res=$this->db->exec($sql, $val);
			if($res===false) {
				return array(
					'success'=>false,
					'txt'=>'db_error'
				);
			}
			$sql="DELETE FROM `receiptlines` WHERE `receipt_id` IN (".$in.")";
			$res=$this->db->exec($sql, $val);
			if($res===false) {
				return array(
					'success'=>false,
					'txt'=>'db_error'
				);
			}
		}

		return array(
			'success'=>true
		);
	}

	// Перемещение чека на другой стол
	function move_receipt($receipt_id, $table_id) {
		if(!$receipt_id || !$table_id) {
			return array(
				'success'=>false,
				'txt'=>'not_enough_data'
			);
		}
		$this->reset();
		$this->load(array('id=?', $receipt_id), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		$t=new \Models\Tables($this->db, $this->f3);
		$t->load(array('id=?', $table_id), array('limit'=>1));
		if($t->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}

		$this->table_id=$table_id;
		if($this->update()) {
			return array(
				'success'=>false,
				'txt'=>'save_successful',
				'function'=>'$("#screen").load("?table_id='.$table_id.' #screen_window");',
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Добавляем интернет-заказ к чеку
	function push_order($receipt_id, $order) {
		$this->reset();
		$this->load(array('id=?', $receipt_id), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		$this->partner_id=$order['partner_id'];
		$client=$order['other']['email'];
		if(!$client) {
			$client=$order['other']['phone'];
		}
		$this->client=$client;
		if($this->update()===false) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}

		if($order['receiptlines']) {
			$lines=new \Models\Receiptlines($this->db, $this->f3);
			foreach ($order['receiptlines'] as $l) {
				$l['line_discount']=$l['discount'];
				$l['discount']=0;
				$l['price0']=$l['price'];
				if($l['line_discount']!=0) {
					$l['price']=$l['price']-($l['price']*($l['line_discount']/100));
				}
				$lines->add_line($receipt_id, $l, $l['count']);
			}
		}

		return array(
			'success'=>true,
			'txt'=>'save_successful',
			'reload'=>'.receiptlines',
			'reload_menu'=>true,
			'close_all_window'=>true
		);

	}

	// Печать пречека или товарного чека
	function print($id) {
		$this->reset();
		$this->load(array('id=?', max($id, -1)), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}

		$receipt=$this->cast();
		$receiptlines=new \Models\Receiptlines($this->db, $this->f3);
		$receipt['receiptlines']=$receiptlines->select_lines(array('receipt_id=?', $receipt['id']));
		if($receipt['time']>0) {
			$pays=json_decode($receipt['payment'], true);
			$p=new \Models\Paymentmethods($this->db, $this->f3);
			$paymentmethods=$p->select_method();
			foreach ($paymentmethods as $p) {
				$methods[$p['id']]=$p['title'];
			}
			foreach ($pays as $n => $p) {
				$pays[$n]['title']=$methods[$p['method']];
			}
			$receipt['pays']=$pays;
			$u=new \Models\Users($this->db, $this->f3);
			$user=$u->select_users(array('u.id=?', $receipt['user_id']), array('limit'=>1));
			$receipt['user']=$user[0];
		}
		$this->fr->print($receipt);
		return array(
			'success'=>true
		);
	}

	// Документы отправленные для удаленной фискализации и успешно фискализированные
	function success_fiscalization($time) {
		$receipts=array();
		$this->reset();
		$this->load(array('remote>0 AND time>=?', $time));
		if(!$this->dry()){
			do {
				$receipts[]=$this->remote;
			}
			while ($this->skip());
		}
		return $receipts;
	}

	// Проверка введенного штрих-кода сотрудника
	function validate_walk($data) {
		$s=new \Models\Settings($this->f3, $this->db);
		$sets=$s->get_settings(array('worker_code_pr'));
		if(!$sets['worker_code_pr']) {
			return array(
				'success'=>false,
				'txt'=>'workers_code_is_not_specified'
			);
		}
		$barcode=trim($data['barcode']);
		if(mb_strlen($barcode)!=13 || mb_substr($barcode, 0, mb_strlen($sets['worker_code_pr']))!=$sets['worker_code_pr'] || !ctype_digit($barcode)) {
			return array(
				'success'=>false,
				'txt'=>'incorrect_workers_code'
			);
		}
		$data['barcode']=$barcode;
		return array(
			'success'=>true,
			'data'=>$data
		);
	}

	// Создаем документ прохода сотрудника
	function add_walk($data) {
		$receipt=array(
			'store'=>$data['store'],
			'rm'=>$data['rm'],
			'user_id'=>0,
			'changes'=>0,
			'changes_data'=>'',
			'time'=>microtime(true),
			'type'=>$data['type'],
			'payment'=>'',
			'sum'=>0,
			'client'=>$data['barcode'],
			'create_receipt'=>0,
			'description'=>$data['action']
		);
		$this->reset();
		$this->copyFrom($receipt);
		if($this->save()===false) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
		return array(
			'success'=>true,
			'txt'=>'save_successful',
			'close_all_window'=>true
		);
	}
}
